<?php
namespace App\Factory;

use App\Exception;

class ConsumerFactory {
    private $connection;
    private $util;

    public function __construct (\Doctrine\DBAL\Connection $connection,
                                  \App\Util\Util $util) {
       $this->connection = $connection;
       $this->util = $util;
    }
    
    public function getConsumer($clientCode, $consumerCode) {
        $sql = "
            SELECT  C.*, M.NMMUNICIPIO
                FROM CONSUMIDOR C
                LEFT JOIN MUNICIPIO M
                    ON M.CDPAIS = C.CDPAIS
                        AND M.SGESTADO = C.SGESTADO
                        AND M.CDMUNICIPIO = C.CDMUNICIPIO
            WHERE C.CDCLIENTE = :CDCLIENTE
             AND C.CDCONSUMIDOR = :CDCONSUMIDOR
        ";
        
        $params = [
            ':CDCLIENTE'    => $clientCode ,
            ':CDCONSUMIDOR' => $consumerCode
        ];
        
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function getGeneralParams($subisidiaryCode) {
        $sql = "
            SELECT F.NMFILIAL, P.CDCLIENTE, P.CDTIPOCONS, P.CDCCUSCLIE, E.SGESTADO, E.CDMUNICIPIO
              FROM PARAVEND P, ENDEFILI E, FILIAL F
               WHERE P.CDFILIAL     = :CDFILIAL
                 AND P.CDFILIAL     = E.CDFILIAL
                 AND F.CDFILIAL     = P.CDFILIAL
                 AND E.IDTPENDEFILI = 'P'
        ";
        
        $params = [
            ':CDFILIAL' => $subisidiaryCode
        ];
        
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function insertConsumer($consumerDetails) {
        $sql = "
            INSERT INTO CONSUMIDOR (
                CDCLIENTE, CDCONSUMIDOR, NMCONSUMIDOR, IDSITCONSUMI,
                CDCCUSCLIE, IDCONSUMIDOR, IDATUCONSUMI, IDTPVENDACONS,
                IDIMPCPFCUPOM, IDCADCONFLIBCON, IDPERCONSPRODEX, IDTPSELMANHA,
                IDTPSEALMOCO, IDTPSELTARDE, IDCRACHAMESTRE, CDPAIS,
                CDTIPOCONS, CDIDCONSUMID, DSINFOCONS, NRTELECONS
            )
            VALUES (
                :CDCLIENTE, :CDCONSUMIDOR, :NMCONSUMIDOR, :IDSITCONSUMI,
                :CDCCUSCLIE, :IDCONSUMIDOR, :IDATUCONSUMI, :IDTPVENDACONS,
                :IDIMPCPFCUPOM, :IDCADCONFLIBCON, :IDPERCONSPRODEX, :IDTPSELMANHA,
                :IDTPSEALMOCO, :IDTPSELTARDE, :IDCRACHAMESTRE, :CDPAIS,
                :CDTIPOCONS, :CDIDCONSUMID, :DSINFOCONS, :NRTELECONS
            )";
        
        $params = [
            ":CDCLIENTE"       => $consumerDetails["CDCLIENTE"],
            ":CDCONSUMIDOR"    => $consumerDetails["CDCONSUMIDOR"],
            ":NMCONSUMIDOR"    => substr($consumerDetails["NMCONSUMIDOR"], 0, 50),
            ":IDSITCONSUMI"    => "1",
            ":CDCCUSCLIE"      => $consumerDetails["CDCCUSCLIE"],
            ":IDCONSUMIDOR"    => "C",
            ":IDATUCONSUMI"    => "S",
            ":IDTPVENDACONS"   => "7",
            ":IDIMPCPFCUPOM"   => "S",
            ":IDCADCONFLIBCON" => "N",
            ":IDPERCONSPRODEX" => "S",
            ":IDTPSELMANHA"    => "N", 
            ":IDTPSEALMOCO"    => "N", 
            ":IDTPSELTARDE"    => "N", 
            ":IDCRACHAMESTRE"  => "N", 
            ":CDPAIS"          => "0055",
            ":CDTIPOCONS"      => $consumerDetails["CDTIPOCONS"],
            ":CDIDCONSUMID"    => $consumerDetails["CDIDCONSUMID"],
            ":DSINFOCONS"      => "Incluído via fidelity-api",
            ":NRTELECONS"      => $consumerDetails["NRTELECONS"]
        ];
        
        $this->connection->executeQuery($sql, $params);
    }
    
    public function getConsumerById($id, $queryField) {
        $sql = "
            SELECT CDCLIENTE, CDCONSUMIDOR
                FROM CONSUMIDOR
                WHERE {$queryField} = :ID
        ";
        
        $params =  [
            ':ID' => $id
        ];
        
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function updateConsumer($data) {
        $sql = "
            UPDATE CONSUMIDOR
                SET 
                    NMCONSUMIDOR    = :NMCONSUMIDOR,
                    DSEMAILCONS     = :DSEMAILCONS,
                    NRTELECONS      = :NRTELECONS,
                    NRCPFRESPCON    = :NRCPFRESPCON,
                    DTNASCCONS      = TO_DATE(:DTNASCCONS, 'DD/MM/YYYY'),
                    IDSEXOCONS      = :IDSEXOCONS,
                    NRCEPCONS       = :NRCEPCONS,
                    DSENDECONS      = :DSENDECONS,
                    NMBAIRCONS      = :NMBAIRCONS,
                    NRENDECONS      = :NRENDECONS,
                    DSCOMPLENDECONS = :DSCOMPLENDECONS,
                    SGESTADO        = :SGESTADO,
                    CDMUNICIPIO     = :CDMUNICIPIO
            WHERE CDCLIENTE = :CDCLIENTE
                AND CDCONSUMIDOR = :CDCONSUMIDOR
        ";
        
        if(!empty($data['gender'])) {
            $data['gender'] = strtoupper($data['gender']) == 'FEMININO' ? 'F' : 'M'; 
        }
        
        $params = [
            ':CDCLIENTE'       => $data['clientCode'],
            ':CDCONSUMIDOR'    => $data['consumerCode'],
            ':NMCONSUMIDOR'    => strtoupper($this->util->removeAcentos(substr($data['name'], 0, 50))),
            ':DSEMAILCONS'     => !empty($data['email']) ? $data['email'] : null,
            ':NRTELECONS'      => !empty($data['phone']) ? preg_replace('/[\(\)\s-]/', '', $data['phone']) : null,
            ':NRCPFRESPCON'    => !empty($data['cpf']) ? preg_replace('/[\.-]/', '', $data['cpf']) : null,
            ':DTNASCCONS'      => !empty($data['birthday']) ? $data['birthday'] : null,
            ':IDSEXOCONS'      => !empty($data['gender']) ? $data['gender'] : null,
            ':NRCEPCONS'       => !empty($data['cep']) ? preg_replace('/[\.-]/', '', $data['cep']) : null,
            ':DSENDECONS'      => !empty($data['street']) ? strtoupper($data['street']) : null,
            ':NMBAIRCONS'      => !empty($data['neighborhood']) ? strtoupper($data['neighborhood']) : null,
            ':NRENDECONS'      => is_numeric($data['addressNumber']) ? $data['addressNumber'] : null,
            ':DSCOMPLENDECONS' => !empty($data['complement']) ? strtoupper($data['complement']) : null,
            ':SGESTADO'        => !empty($data['state']) ? strtoupper($data['state']) : null,
            ':CDMUNICIPIO'     => isset($data['CDMUNICIPIO']) ? $data['CDMUNICIPIO'] : null
        ];
        
        $this->connection->executeQuery($sql, $params);
    }
    
    public function getMunicipio($sgEstado, $nmMunicipio) {
        $sql = "
            SELECT CDMUNICIPIO FROM MUNICIPIO
                WHERE SGESTADO = :SGESTADO
                 AND NMMUNICIPIO = :NMMUNICIPIO
        ";
        
        $params = [
            ':SGESTADO'    => strtoupper($sgEstado),
            ':NMMUNICIPIO' => strtoupper($nmMunicipio)
        ];
        
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function updateConsumerPhone($cdCliente, $cdConsumidor, $nrTeleCons) {
        $sql = "
            UPDATE CONSUMIDOR
                SET 
                    NRTELECONS = :NRTELECONS
            WHERE CDCLIENTE = :CDCLIENTE
              AND CDCONSUMIDOR = :CDCONSUMIDOR
        ";
        
        $params = [
            ':CDCLIENTE'    => $cdCliente,
            ':CDCONSUMIDOR' => $cdConsumidor,
            ':NRTELECONS'   => $nrTeleCons
        ];
        
        return $this->connection->executeQuery($sql, $params);
    }
}